import { Keypair, Transaction, } from '@solana/web3.js';
import { toChainId, toNative, } from '@wormhole-foundation/connect-sdk';
import { SolanaAddress, SolanaPlatform, SolanaUnsignedTransaction, } from '@wormhole-foundation/connect-sdk-solana';
import { createPostVaaInstruction, createReadOnlyWormholeProgramInterface, createVerifySignaturesInstructions, } from './utils';
export class SolanaWormholeCore {
    network;
    chain;
    connection;
    contracts;
    chainId;
    coreBridge;
    address;
    constructor(network, chain, connection, contracts) {
        this.network = network;
        this.chain = chain;
        this.connection = connection;
        this.contracts = contracts;
        this.chainId = toChainId(chain);
        const coreBridgeAddress = contracts.coreBridge;
        if (!coreBridgeAddress)
            throw new Error(`CoreBridge contract Address for chain ${chain} not found`);
        this.address = coreBridgeAddress;
        this.coreBridge = createReadOnlyWormholeProgramInterface(coreBridgeAddress, connection);
    }
    static async fromRpc(connection, config) {
        const [network, chain] = await SolanaPlatform.chainFromRpc(connection);
        const conf = config[chain];
        if (conf.network !== network)
            throw new Error(`Network mismatch for chain ${chain}: ${conf.network} != ${network}`);
        return new SolanaWormholeCore(network, chain, connection, config[chain].contracts);
    }
    publishMessage(sender, message) {
        throw new Error('Method not implemented.');
    }
    async *postVaa(sender, vaa, blockhash) {
        const senderAddr = new SolanaAddress(sender).unwrap();
        const signatureSet = Keypair.generate();
        const verifySignaturesInstructions = await createVerifySignaturesInstructions(this.connection, this.coreBridge.programId, senderAddr, vaa, signatureSet.publicKey);
        // Create a new transaction for every 2 signatures we have to Verify
        for (let i = 0; i < verifySignaturesInstructions.length; i += 2) {
            const verifySigTx = new Transaction().add(...verifySignaturesInstructions.slice(i, i + 2));
            verifySigTx.recentBlockhash = blockhash;
            verifySigTx.feePayer = senderAddr;
            verifySigTx.partialSign(signatureSet);
            yield this.createUnsignedTx(verifySigTx, 'Redeem.VerifySignature', true);
        }
        // Finally create the VAA posting transaction
        const postVaaTx = new Transaction().add(createPostVaaInstruction(this.connection, this.coreBridge.programId, senderAddr, vaa, signatureSet.publicKey));
        postVaaTx.recentBlockhash = blockhash;
        postVaaTx.feePayer = senderAddr;
        yield this.createUnsignedTx(postVaaTx, 'Redeem.PostVAA');
    }
    async parseTransaction(txid) {
        const response = await this.connection.getTransaction(txid, {
            maxSupportedTransactionVersion: 0,
        });
        if (!response || !response.meta || !response.meta.innerInstructions)
            throw new Error('transaction not found');
        // Resolve any LUT accounts if necessary
        let accounts;
        // a string type is indicative of a 'legacy' transaction
        if (typeof response.transaction.message.version !== 'string') {
            if (response.meta.loadedAddresses) {
                accounts = response.transaction.message.getAccountKeys({
                    accountKeysFromLookups: response.meta.loadedAddresses,
                });
            }
            else {
                const atls = await Promise.all(response.transaction.message.addressTableLookups.map(async (atl) => {
                    const lut = await this.connection.getAddressLookupTable(atl.accountKey);
                    if (!lut || !lut.value)
                        throw new Error('Could not resolve lookup table: ' + atl.accountKey.toBase58());
                    return lut.value;
                }));
                accounts = response.transaction.message.getAccountKeys({
                    addressLookupTableAccounts: atls,
                });
            }
        }
        else {
            accounts = response.transaction.message.getAccountKeys();
        }
        // find the instruction where the programId equals the Wormhole ProgramId
        const iix = response.meta.innerInstructions[0].instructions.filter((i) => {
            const programId = accounts.get(i.programIdIndex).toString();
            const wormholeCore = this.coreBridge.programId.toString();
            console.log(programId, wormholeCore);
            return programId === wormholeCore;
        }).map((ix) => {
            // map over inner ixs to put it in the same format as the compiled instructions
            // from the message
            return {
                programIdIndex: ix.programIdIndex,
                accountKeyIndexes: ix.accounts,
            };
        });
        const cix = response.transaction.message.compiledInstructions.filter((i) => {
            const programId = accounts.get(i.programIdIndex).toString();
            const wormholeCore = this.coreBridge.programId.toString();
            console.log(programId, wormholeCore);
            return programId === wormholeCore;
        });
        // find the instruction where the programId equals the Wormhole ProgramId
        const bridgeInstructions = [...iix, ...cix];
        if (!bridgeInstructions || bridgeInstructions.length === 0)
            throw new Error('no bridge messages found');
        const messagePromises = bridgeInstructions.map(async (bi) => {
            const messageAcct = accounts.get(bi.accountKeyIndexes[1]);
            const acctInfo = await this.connection.getAccountInfo(messageAcct);
            const sequence = acctInfo.data.readBigUInt64LE(49);
            const emitterAddr = new Uint8Array(acctInfo.data.subarray(59, 91));
            const emitter = toNative(this.chain, emitterAddr);
            return {
                chain: this.chain,
                emitter: emitter.toUniversalAddress(),
                sequence: BigInt(sequence),
            };
        });
        return await Promise.all(messagePromises);
    }
    createUnsignedTx(txReq, description, parallelizable = false) {
        return new SolanaUnsignedTransaction(txReq, this.network, this.chain, description, parallelizable);
    }
}
//# sourceMappingURL=core.js.map